/*
 * Decompiled with CFR 0_101.
 */
package com.denova.io;

import com.denova.io.TempFiles;
import com.denova.ui.TextDisplayable;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.io.RandomAccessFile;
import java.io.StringWriter;
import java.io.Writer;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.StringTokenizer;
import java.util.TimeZone;

public class Log {
    private static final boolean debugging = false;
    private static final String ExceptionMessage = "Exception: ";
    private static final String StdLogSuffix = ".log";
    private static boolean alwaysFlush = false;
    private static boolean useTempDir = false;
    private static File loggingDir = null;
    private static String logSuffix = ".log";
    private RandomAccessFile out;
    private String logFilename;
    private boolean logging;
    private boolean enabled;
    private TextDisplayable statusDisplay;
    private Log log2 = null;

    public Log(String string, String string2) {
        File file = new File(string);
        this.initLog(file, string2);
    }

    public Log(File file, String string) {
        this.initLog(file, string);
    }

    public Log(String string) {
        if (loggingDir == null) {
            loggingDir = this.getDir();
        }
        this.initLog(string);
    }

    public static void setLogDir(String string) {
        if (string == null || string.length() <= 0) {
            string = System.getProperty("user.dir", ".");
        }
        loggingDir = new File(string);
    }

    public static void setLogDir(File file) {
        if (file == null) {
            file = new File(System.getProperty("user.dir", "."));
        }
        Log.setLogDir(file.getPath());
    }

    public static String getLogDir() {
        String string = null;
        if (loggingDir != null) {
            string = loggingDir.getPath();
        }
        return string;
    }

    public static void setAlwaysFlush(boolean bl) {
        alwaysFlush = bl;
    }

    public static boolean getAlwaysFlush() {
        return alwaysFlush;
    }

    public static void setUseTempDirectory(boolean bl) {
        useTempDir = bl;
    }

    public static boolean getUseTempDirectory() {
        return useTempDir;
    }

    public void end() {
        this.stopLogging();
    }

    public synchronized void write(String string) {
        this.writeMessage(string);
    }

    public synchronized void write(Exception exception) {
        this.logThrowable(exception);
    }

    public synchronized void write(Throwable throwable) {
        this.logThrowable(throwable);
    }

    public synchronized void write(Throwable throwable, String string) {
        this.write(string);
        this.logThrowable(throwable);
    }

    public boolean getLogging() {
        return this.logging;
    }

    public void setLogging(boolean bl) {
        if (bl) {
            this.enabled = true;
        } else {
            this.disableLogging();
        }
    }

    public TextDisplayable getStatusDisplay() {
        return this.statusDisplay;
    }

    public void setStatusDisplay(TextDisplayable textDisplayable) {
        this.statusDisplay = textDisplayable;
    }

    public Log getAlsoLogTo() {
        return this.log2;
    }

    public void setAlsoLogTo(Log log) {
        this.log2 = log;
    }

    public void startLogging() {
        this.enableLogging();
    }

    public void stopLogging() {
        this.disableLogging();
    }

    public void flush() {
        this.forceWrite();
    }

    public boolean logFileExists() {
        File file = new File(this.logFilename);
        return file.exists();
    }

    public boolean logFileHasContent() {
        boolean bl = false;
        if (this.logFileExists()) {
            File file = new File(this.logFilename);
            bl = file.length() > 1;
        }
        return bl;
    }

    public void reset() {
        this.debug("resetting: " + this.logFilename);
        if (this.logFileHasContent()) {
            boolean bl = this.logging;
            this.debug("originally logging: " + bl);
            TextDisplayable textDisplayable = this.statusDisplay;
            this.statusDisplay = null;
            this.disableLogging();
            File file = new File(this.logFilename);
            file.delete();
            if (bl) {
                this.enableLogging();
            }
            this.debug("logging: " + this.logging);
            this.statusDisplay = textDisplayable;
        }
        this.debug("finished resetting log");
    }

    public void setLogExt(String string) {
        logSuffix = string;
    }

    public String getLogExt() {
        return logSuffix;
    }

    private void writeMessage(String string) {
        if (this.enabled) {
            if (!this.logging) {
                this.enableLogging();
            }
            if (this.logging || this.log2 != null) {
                this.writeData(string);
            }
            if (alwaysFlush) {
                this.forceWrite();
            }
            if (this.statusDisplay != null) {
                this.statusDisplay.displayText(string);
            }
        }
    }

    private void writeData(String string) {
        TimeZone timeZone = TimeZone.getTimeZone("GMT");
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
        simpleDateFormat.setTimeZone(timeZone);
        Date date = new Date();
        StringBuffer stringBuffer = new StringBuffer(simpleDateFormat.format(date));
        stringBuffer.append(": ");
        stringBuffer.append("[" + Thread.currentThread().toString() + "] ");
        stringBuffer.append(string);
        stringBuffer.append(System.getProperty("line.separator"));
        if (this.logging) {
            this.output(stringBuffer.toString());
        }
        if (this.log2 != null) {
            this.log2.write(stringBuffer.toString());
        }
    }

    private void forceWrite() {
        if (this.enabled && this.logging) {
            try {
                this.out.close();
                this.openLog();
            }
            catch (Exception var1_1) {
                System.err.println("Exception: " + var1_1);
                var1_1.printStackTrace();
            }
        }
    }

    private File getDir() {
        File file = null;
        if (useTempDir) {
            file = TempFiles.getDefaultDirectory();
            this.debug("using temp dir " + file);
        }
        if (file == null || file.getPath().length() <= 0) {
            file = new File(System.getProperty("user.dir", "."));
            this.debug("using user.dir " + file);
        }
        return file;
    }

    private void initLog(File file, String string) {
        File file2 = null;
        boolean bl = false;
        if (file == null || file.getPath().length() <= 0) {
            loggingDir = this.getDir();
        } else {
            file2 = loggingDir;
            loggingDir = file;
            bl = true;
        }
        this.initLog(string);
        if (bl) {
            loggingDir = file2;
        }
    }

    private void initLog(String string) {
        StringBuffer stringBuffer = new StringBuffer(string);
        if (!string.endsWith(logSuffix)) {
            stringBuffer.append(logSuffix);
        }
        File file = new File(loggingDir, stringBuffer.toString());
        try {
            this.logFilename = file.getAbsolutePath();
            this.debug("initing absolute path for log " + this.logFilename);
        }
        catch (Exception var4_4) {
            this.logFilename = file.getPath();
            this.debug("initing log " + this.logFilename);
        }
        boolean bl = this.logFileExists();
        this.debug("log exists " + String.valueOf(bl));
        this.setLogging(bl);
    }

    private void enableLogging() {
        if (!this.logging) {
            try {
                this.openLog();
                this.logging = true;
                this.enabled = true;
                this.output(System.getProperty("line.separator"));
            }
            catch (IOException var1_1) {
                System.err.println("Exception: " + var1_1);
                var1_1.printStackTrace();
            }
            if (!this.logging) {
                try {
                    this.out.close();
                }
                catch (Exception var1_2) {
                    // empty catch block
                }
            }
        }
    }

    private void disableLogging() {
        if (this.logging) {
            this.writeMessage("Ended logging");
            try {
                this.out.close();
            }
            catch (IOException var1_1) {
                System.err.println("Exception: " + var1_1);
                var1_1.printStackTrace();
            }
            this.logging = false;
        }
        this.enabled = false;
    }

    private void openLog() throws FileNotFoundException, IOException {
        File file = new File(this.logFilename);
        this.out = new RandomAccessFile(file, "rw");
        this.out.seek(this.out.length());
    }

    private void output(String string) {
        try {
            this.out.writeBytes(string);
        }
        catch (IOException var2_2) {
            System.err.println("Exception: " + var2_2);
            var2_2.printStackTrace();
        }
    }

    private void logThrowable(Exception exception) {
        if (exception != null) {
            StringWriter stringWriter = new StringWriter();
            PrintWriter printWriter = new PrintWriter(stringWriter);
            exception.printStackTrace(printWriter);
            StringTokenizer stringTokenizer = new StringTokenizer(stringWriter.toString(), "\r\n");
            while (stringTokenizer.hasMoreTokens()) {
                String string = stringTokenizer.nextToken();
                this.write(string);
            }
            printWriter.close();
        }
    }

    private void logThrowable(Throwable throwable) {
        if (throwable != null) {
            StringWriter stringWriter = new StringWriter();
            PrintWriter printWriter = new PrintWriter(stringWriter);
            throwable.printStackTrace(printWriter);
            StringTokenizer stringTokenizer = new StringTokenizer(stringWriter.toString(), "\r\n");
            while (stringTokenizer.hasMoreTokens()) {
                String string = stringTokenizer.nextToken();
                this.write(string);
            }
            printWriter.close();
        }
    }

    private void debug(String string) {
    }
}
